﻿// The MIT License (MIT)
// 
// COPYRIGHT 2013. BLOOMBERG FINANCE L.P.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Ipc;
using System.Runtime.Serialization.Formatters;

namespace PluginHost.Hosting
{
    public class HostProcess : MarshalByRefObject
    {
        public PluginController Controller { get; set; }

        public HostProcess()
        {
            var channel = new IpcServerChannel(
                new ListDictionary {{"portName", RemotingConfiguration.ProcessId}},
                new BinaryServerFormatterSinkProvider {TypeFilterLevel = TypeFilterLevel.Full});

            ChannelServices.RegisterChannel(channel, false);

            string[] args =
                {
                    Process.GetCurrentProcess().Id.ToString(CultureInfo.InvariantCulture),
                    channel.GetUrlsForUri(RemotingServices.Marshal(this).URI)[0]
                };

            Process.Start(
                new ProcessStartInfo
                    {
                        FileName = "PluginHost.Hosting.Host.exe",
                        Arguments = string.Join(" ", args),
                        UseShellExecute = false,
                        CreateNoWindow = true,
                    });
        }
    }
}