﻿// The MIT License (MIT)
// 
// COPYRIGHT 2013. BLOOMBERG FINANCE L.P.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Windows;
using System.Xml.XPath;

namespace PluginHost.Hosting
{
    public class PluginContainer : MarshalByRefObject
    {
        private string[] _args;

        public void Launch(string[] args)
        {
            _args = args;

            var thread = new Thread(Run);
            thread.TrySetApartmentState(ApartmentState.STA);
            thread.Start();
        }

        private void Run()
        {
            var metadata = new XPathDocument(
                Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Metadata.xml"))
                .CreateNavigator().SelectSingleNode("/Plugin");

            Debug.Assert(metadata != null);

            var mainAssembly = (string) metadata.Evaluate("string(MainAssembly)");
            var mainClass = (string) metadata.Evaluate("string(MainClass)");
            var title = (string) metadata.Evaluate("string(Title)");

            Debug.Assert(!string.IsNullOrEmpty(mainAssembly));
            Debug.Assert(!string.IsNullOrEmpty(mainClass));
            Debug.Assert(!string.IsNullOrEmpty(title));

            var rootElement = ((Func<string[], UIElement>) Delegate.CreateDelegate(
                typeof (Func<string[], UIElement>),
                Assembly.Load(mainAssembly).GetType(mainClass),
                "CreateRootElement"))(_args);

            var window = 
                new Window
                    {
                        SizeToContent = SizeToContent.WidthAndHeight,
                        Title = title,
                        Content = rootElement
                    };

            new Application().Run(window);
            AppDomain.Unload(AppDomain.CurrentDomain);
        }
    }
}
