﻿// The MIT License (MIT)
// 
// COPYRIGHT 2013. BLOOMBERG FINANCE L.P.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;

namespace PluginHost.Hosting
{
    public class PluginController : MarshalByRefObject
    {
        private readonly string _appsRoot;

        public PluginController()
        {
            _appsRoot = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Plugins");
        }

        public void Launch(string commandLine)
        {
            var tokens = commandLine.Split(
                new[] {' '}, StringSplitOptions.RemoveEmptyEntries);

            var mnemonic = tokens[0];
            if (mnemonic.Length > 4)
            {
                mnemonic = mnemonic.Substring(0, 4);
            }

            var args = new string[tokens.Length - 1];
            Array.Copy(tokens, 1, args, 0, args.Length);

            var appPath = Path.Combine(_appsRoot, mnemonic);

            var setup = new AppDomainSetup {ApplicationBase = appPath};
            var config = new RuntimeConfig();

            var pluginContType = typeof(PluginContainer);
            var assemblyName = pluginContType.Assembly.GetName();

            config.AddCodeBaseFor(assemblyName);

            foreach (var name in Directory.GetFiles("PAC", "*.dll")
                .Select(AssemblyName.GetAssemblyName))
            {
                config.AddCodeBaseFor(name);
            }
            setup.SetConfigurationBytes(config.GetBytes());

            var domain = AppDomain.CreateDomain(mnemonic, null, setup);

            var host = (PluginContainer) domain.CreateInstanceAndUnwrap(
                pluginContType.Assembly.FullName, pluginContType.FullName);

            host.Launch(args);
        }

        public override object InitializeLifetimeService()
        {
            return null;
        }
    }
}