﻿// The MIT License (MIT)
// 
// COPYRIGHT 2013. BLOOMBERG FINANCE L.P.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls.DataVisualization.Charting;

namespace DapiSample
{
    /// <summary>
    /// Interaction logic for MainView.xaml
    /// </summary>
    public partial class MainView
    {
        private readonly MarketDataProvider _marketDataProvider;

        public MainView()
        {
            InitializeComponent();
            _chart.Visibility = Visibility.Hidden;

            // set series style
           var st = new Style(typeof(LineDataPoint));
            st.Setters.Add(new Setter(TemplateProperty, null));
            _lineSeries.DataPointStyle = st;

            _marketDataProvider = new MarketDataProvider();
            _marketDataProvider.MarketDataEventHandler += OnMarketDataHandler;
            _marketDataProvider.MarketDataErrorHandler += OnMarketDataError;

            this.Unloaded += (sender, args) => { _marketDataProvider.Dispose(); };
        }

        public MainView(string[] args) : this()
        {
            _security.Text = string.Join(" ", args);

            if (_security.Text == string.Empty)
            {
                return;
            }
            RequestData();
        }

        private void ButtonClick(object sender, RoutedEventArgs e)
        {
            RequestData();
        }

        private void RequestData()
        {
            _marketDataProvider.RequestReferenceData(_security.Text, new DateTime(DateTime.Now.Year, 1, 1), DateTime.Now, "DAILY");
        }
        
        private void OnMarketDataError(string error)
        {
            Dispatcher.BeginInvoke((Action)delegate
            {
                try
                {
                    MessageBox.Show(error, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.ToString());
                }

            });

        }

        private void OnMarketDataHandler(List<DataPoint> series)
        {
            Dispatcher.BeginInvoke((Action)delegate
            {
                try
                {
                    _chart.Visibility = Visibility.Hidden;
                    SetLinearAxis(series);
                    _chart.DataContext = series;
                    _chart.Visibility = Visibility.Visible;
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.ToString());
                }

            });
        }

        private void SetLinearAxis(List<DataPoint> series)
        {
            _linearAxis.Minimum = 0;

            double max = series.Max(val => val.Value);
            double min = series.Min(val => val.Value);

            double range = (int)(max - min);
            int intervals;

            if (range < 100)
                intervals = 5;
            else
            {
                intervals = 8;
            }

            double interval = range / intervals;

            if (interval < 1)
                interval = .5;
            else
                interval = (int)interval;


            _linearAxis.Maximum = max;
            _linearAxis.Minimum = min;

            _linearAxis.Interval = interval;
            
        }
    }
}
